#include "reg52.h"  
#include "onewire.h"

sbit HC138_A = P2^5;	
sbit HC138_B = P2^6;	
sbit HC138_C = P2^7;	

unsigned char code SMG_NoDot[18] = {0xc0,0xf9,0xa4,
	0xb0,0x99,0x92,0x82,0xf8,0x80,0x90,0x88,0x80,0xc6,
	0xc0,0x86,0x8e,0xbf,0x7f};

unsigned char code SMG_IsDot[10] = {0x40,0x79,0x24,
	0x30,0x19,0x12,0x02,0x78,0x00,0x10};

unsigned int T_dat;

void Delay(unsigned int t);
void DelaySMG(unsigned int time);
void Init74HC138(unsigned char n);
void DisplaySMG_Temperature(unsigned int dat);
void DisplaySMG_ALL(unsigned char value);
void Read_Temperature();

void main()
{
	while(1)
	{
		Read_Temperature();
		DisplaySMG_Temperature(T_dat);
	}
}

void Delay(unsigned int t)
{
	while(t--)
	{
		DisplaySMG_Temperature(T_dat);	
	}
}

void DelaySMG(unsigned int time)
{
	while(time--);
}

void Init74HC138(unsigned char n)
{
	switch(n)
	{
		case 4:
			HC138_A = 0;
			HC138_B = 0;
			HC138_C = 1;
			break;
		case 5:
			HC138_A = 1;
			HC138_B = 0;
			HC138_C = 1;
			break;
		case 6:
			HC138_A = 0;
			HC138_B = 1;
			HC138_C = 1;
			break;
		case 7:
			HC138_A = 1;
			HC138_B = 1;
			HC138_C = 1;
			break;
		case 8:
			HC138_A = 0;
			HC138_B = 0;
			HC138_C = 0;
			break;
	}
}

void DisplaySMG_Bit(unsigned char pos, unsigned char value)
{
	Init74HC138(7);
	P0 = 0xff;
	Init74HC138(6);
	P0 = (0x01 << pos);
	Init74HC138(7);
	P0 = value;
}

void DisplaySMG_ALL(unsigned char value)
{
	Init74HC138(7);
	P0 = value;
	Init74HC138(6);
	P0 = 0xff;
}

void DisplaySMG_Temperature(unsigned int dat)
{
	DisplaySMG_Bit(7, SMG_NoDot[dat%10]);
	DelaySMG(100);
	DisplaySMG_Bit(6, SMG_IsDot[(dat/10)%10]);
	DelaySMG(100);
	DisplaySMG_Bit(5, SMG_NoDot[(dat/100)%10]);
	DelaySMG(100);
	
	DisplaySMG_ALL(0xff);
}

void Read_Temperature()
{
	unsigned char LSB,MSB ;
	
	Init_DS18B20();			
	Write_DS18B20(0xCC);			
	Write_DS18B20(0x44);
	Delay(1000);
	Init_DS18B20();		
	Write_DS18B20(0xCC);		
	Write_DS18B20(0xBE);			
	LSB = Read_DS18B20();		
	MSB = Read_DS18B20();			
	Init_DS18B20();		

	T_dat = 0x0000;
	T_dat = MSB;
	T_dat <<= 8;
	T_dat = T_dat | LSB;
	
	if((T_dat & 0xf800) == 0x0000)
	{
		T_dat >>= 4;
		T_dat = T_dat*10;
		T_dat = T_dat + (LSB&0x0f)*0.625;
	}
}